// -*-C++-*-
//
// File:      Dll.h
// Author:    jbakker
// Generated: Wed Apr 26 13:39:09 MET DST 2000

#ifndef Dll_h
#define Dll_h

#include <string>

/*
  Provides an interface to use dynamic linked libraries at runtime

  Example:
    // /usr/local/libFoo.so: int doSomething(int);

    Dll dll("/usr/local/libFoo.so");
    if (dll.valid()) {
      typedef int (*DLLFUNCTION)(int);
      DLLFUNCTION pDoSomething=(DLLFUNCTION)dll.function("doSomething");
      if (pDoSomething) {
         cout << (*pDoSomething)(123) << endl;
      }
    }
  
 */
class Dll
{
public:
  Dll();

  // constructor that opens a library
  Dll(const string& library);
  virtual ~Dll();

  // open a library.
  bool open(const string& library);
  void close();

  // opens the library, returns NULL if function does not exist
  void *function(const string& name);

  bool valid() {return (_errmsg.empty());}

  // returns last error
  const string& error() {return _errmsg;}

protected:
  // only derived classes can use these

private:
  // member functions
  Dll(const Dll&);            // copy constructor (disabled)
  Dll& operator=(const Dll&); // assignment (disabled)

  // member data
  string _name;
  string _errmsg;
  void  *_handle;
};



#endif
